proc baseconvert {base_from base_to number} {
     set number [string tolower $number]
     if {![regexp {([0-9a-z]*)\.?([0-9a-z]*)} $number match sint sfra]} {
         puts "baseconvert error: number \"$number\" is not in correct format"
         return ""
     }
     set map 0123456789abcdefghijklmnopqrstuvwxyz
     set i -1
     foreach c [split $map ""] {
         incr i
         set D2I($c) $i
         set I2D($i) $c
     }
     set lint [string length $sint]
     set lfra [string length $sfra]
     set converted_number 0
     if {$lint > 0} {
         set i -1
         foreach c [split $sint ""] {
             set B([incr i]) $D2I($c)
         }
         set aint ""
         while {1} {
             set s 0
             set r 0
             for {set i 0} {$i < $lint} {incr i} {
                 set v [expr $B($i) + $r*$base_from]
                 set B($i) [expr int($v/$base_to)]
                 set r [expr $v - $base_to*$B($i)]
                 set s [expr $s + $B($i)]
             }
             set aint "$I2D($r)$aint"
             if {$s == 0} {break}
         }
         set converted_number $aint
     }
     if {$lfra > 0} {
         set s [expr $lfra*int(1.0*log($base_from)/log($base_to))]
         set i $lfra
         foreach c [split $sfra ""] {
             set B([incr i -1]) $D2I($c)
         }
         set afra ""
         for {set j 0} {$j < $s} {incr j} {
             set r 0
             for {set i 0} {$i < $lfra} {incr i} {
                 set v [expr $base_to*$B($i) + $r]
                 set r [expr int($v/$base_from)]
                 set B($i) [expr $v - $r*$base_from]
             }
             set afra "$I2D($r)$afra"
         }
         append converted_number .$afra
     }
     return $converted_number
 }

proc console_write {output_file output_string} {
  puts -nonewline $output_string
  puts -nonewline $output_file $output_string
}

# List all available programming hardwares, and select the USBBlaster.
# (Note: this example assumes only one USBBlaster connected.)
# puts "Programming devices:"
foreach hardware_name [get_hardware_names] {
	# puts $hardware_name
	if { [string match "USB-Blaster*" $hardware_name] } {
		set usbblaster_name $hardware_name
	}
}
# puts "\nUsing JTAG chain connected to $usbblaster_name.\n";


# List all devices on the chain, and select the first device on the chain.
# puts "\nDevices on the JTAG chain:"
foreach device_name [get_device_names -hardware_name $usbblaster_name] {
	# puts $device_name
	if { [string match "@1*" $device_name] } {
		set test_device $device_name
	}
}
# puts "\nSelected device: $test_device.\n";

# Open device 
open_device -hardware_name $usbblaster_name -device_name $test_device

# Retrieve device id code.
# IDCODE instruction value is 6; The ID code is 32 bits long.

# IR and DR shift should be locked together to ensure that other applications 
# will not change the instruction register before the id code value is shifted
# out while the instruction register is still holding the IDCODE instruction.
device_lock -timeout 10000
device_ir_shift -ir_value 6 -no_captured_ir_value
# puts "IDCODE: 0x[device_dr_shift -length 32 -value_in_hex]"
device_unlock

set button_name [ lindex $argv 0 ]
# puts -nonewline "type the name of a button: "
# gets stdin button_name

set suffix ".csv"
set output_file [ open $button_name$suffix "w" ]
set output_string "button name,$button_name\n"
console_write $output_file $output_string

set clock_period 20e-9
set output_string "clock period,$clock_period\n"
console_write $output_file $output_string

# The number of actual data lines collected will be half of the amount
# below, since about half of the lines will be inter-transmission gaps.
set data_lines 100

while {$data_lines > 0} {

  while {$data_lines > 0} {
    # read-empty read:
    set ir_value_load 0
    set dr_width 1
    set instance 0

    device_lock -timeout 10000
    device_virtual_ir_shift -instance_index $instance -ir_value $ir_value_load
    set rdempty [ device_virtual_dr_shift -instance_index $instance -length $dr_width -dr_value "0" ]
    device_unlock

    if { $rdempty == 1 } {
      #puts "empty fifo ($rdempty)"
      break
    }

    # rdusedw
  #      set ir_value_load 1
  #      set dr_width 6
  #      set instance 0
  #
  #      device_lock -timeout 10000
  #      device_virtual_ir_shift -instance_index $instance -ir_value $ir_value_load
  #      set rdusedw [ device_virtual_dr_shift -instance_index $instance -length $dr_width -dr_value "000000" ]
  #      device_unlock
  #      puts "rdusedw: $rdusedw"

    # readdata
    set ir_value_load 2
    set dr_width 20
    set instance 0

    device_lock -timeout 10000
    device_virtual_ir_shift -instance_index $instance -ir_value $ir_value_load
    set readdata [ device_virtual_dr_shift -instance_index $instance -length $dr_width -dr_value "00000000000000000000" ]
    device_unlock

    set dec [ baseconvert 2 10 $readdata ]
    # set hex [ baseconvert 2 16 $readdata ]
    # set ms [expr $dec * 20e-9 * 1000]

    set clock_period_us [expr $clock_period * 1000 ]
    set large_period_ms 4
    set threshold [ expr int($large_period_ms / $clock_period_us) ]
    if { $dec > $threshold } {
      set prefix "\n"
      incr data_lines -1
    } else {
      set prefix ""
    }
    set output_string "$prefix$dec, "
    console_write $output_file $output_string
  }
  after 50
}

close_device

close $output_file
