@rem = '--*-Perl-*--
@echo off
if "%OS%" == "Windows_NT" goto WinNT
perl -x -S "%0" %1 %2 %3 %4 %5 %6 %7 %8 %9
goto endofperl
:WinNT
perl -x -S %0 %*
if NOT "%COMSPEC%" == "%SystemRoot%\system32\cmd.exe" goto endofperl
if %errorlevel% == 9009 echo You do not have Perl in your PATH.
if errorlevel 1 goto script_failed_so_exit_with_non_zero_val 2>nul
goto endofperl
@rem ';
#!perl
#line 15

use strict;

sub in_bin($$)
{
  my ($bin, $value) = @_;

  my $in_bin = 0;
  if (($value < $bin->{max}) && ($value > $bin->{min}))
  {
    $in_bin = 1;
  }

  return $in_bin;
}

my %bins = (
  mark_4ms => {
    min => 4.01,
    max => 4.07,
    histogram_data => {},
    filename => "bin_mark_4ms.csv",
    data => [],
  },
  space_4ms => {
    min => 3.9,
    max => 4.0,
    histogram_data => {},
    filename => "bin_space_4ms.csv",
    data => [],
  },
  mark_500us => {
    min => 0.5,
    max => 0.56,
    histogram_data => {},
    filename => "bin_mark_500us.csv",
    data => [],
  },
  space_1ms => {
    min => 0.9,
    max => 1.0,
    histogram_data => {},
    filename => "bin_space_1ms.csv",
    data => [],
  },
  space_2ms => {
    min => 1.94,
    max => 2.0,
    histogram_data => {},
    filename => "bin_space_2ms.csv",
    data => [],
  },
);

# This routine expects a list reference of \n-terminated strings,
# as produced by get_data.tcl.
# The routine returns 3 list reference of \n-terminated strings:
# 1) the "cooked" data (suitable for writing to a csv file),
# 2) any "info" messages (maybe for stdout)
# 3) any "error" messages (maybe for stderr)
# 4) a summary, which could appended to a report for all inputs.
sub transform_data($)
{
  my ($input_data) = @_;

  my $output_data = [];
  my $info = [];
  my $error = [];
  my $summary = {};

  my @raw_data;
  my @cooked_data;
  my $button_name;
  my $clock_period;
  for (@$input_data)
  {
    my $line = $_;
    if ($line =~ /^$/)
    {
    }
    elsif ($line =~ /^button name,(.+)$/)
    {
      $button_name = $1;
    }
    elsif ($line =~ /^clock period,(.*)$/)
    {
      $clock_period = eval($1);
    }
    else
    {
      my @numbers = split(/\s*,\s*/, $line);
      if (@numbers > 1)
      {
        push @raw_data, [@numbers];
      }
    }
  }

  # A bit of validation: was the input data sane?
  my $good_input = 1;
  if (!defined $button_name)
  {
    push @$error, "no button name\n";
    $good_input = 0;
  }

  if (!$clock_period)
  {
    push @$error, "no clock period\n";
    $good_input = 0;
  }

  if (!@raw_data)
  {
    push @$error, "no data\n";
    $good_input = 0;
  }

  if ($good_input)
  {
    # Verify that each data record has the same number of numbers.
    # But one extra item is possible: if the inter-transmission value
    # wasn't large enough to cause get_data.tcl to precede it with \n.
    my $size = 0 + @{$raw_data[0]};

    for my $vector (@raw_data)
    {
      my $this_size = 0 + @$vector;
      if ($this_size == $size + 1)
      {
        push @$info, "warning: one extra item: ($size, $this_size); ";
        push @$info, "ignoring the problem\n";

        # Fix it by deleting the extra item.
        pop @$vector;
      }
    }

    push @$output_data, "button name,$button_name\n";
    push @$output_data, "clock period,$clock_period\n\n";

    push @$output_data, "raw data\n";
    for my $vector (@raw_data)
    {
      push @$output_data, ", ", join(", ", @$vector), "\n";
    }
    push @$output_data, "\n";

    # Multiply each vector component by the clock period.
    # That gives me data in units of seconds.  Scale up to ms 
    # for easy handling. Trivial point: a raw value of "0" means
    # 20ns.  So increment raw values before scaling.
    for my $vector (@raw_data)
    {
      push @cooked_data, [@$vector];
    }

    for my $vector (@cooked_data)
    {
      for my $component (@$vector)
      {
        $component++; # "0" -> 20ns
        $component *= $clock_period * 1000;
      }
    }

    push @$output_data, "cooked data\n";
    for my $vector (@cooked_data)
    {
      push @$output_data, ", ", join(", ", @$vector), "\n";
    }
    push @$output_data, "\n";

    # For each vector component, accumulate average, min, max.
    my @avg = (0.0) x $size;
    my @min = (1e6) x $size;
    my @max = (0) x $size; 
    for my $vector (@cooked_data)
    {
      for my $i (0 .. $size - 1)
      {
        $avg[$i] += $vector->[$i];

         if ($min[$i] > $vector->[$i]) 
         {
           $min[$i] = $vector->[$i];
         }
         if ($max[$i] < $vector->[$i])
         {
           $max[$i] = $vector->[$i];
         }
      }
    }

    my @min_dev = (0) x $size;
    my @max_dev = (0) x $size;
    for my $i (0 .. $size - 1)
    {
      $avg[$i] /= scalar(@cooked_data);
      $min_dev[$i] = $min[$i] / $avg[$i];
      $max_dev[$i] = $max[$i] / $avg[$i];
    }

    sub format_list($$@)
    {
      my ($caption, $size, @list) = @_;
      return "$caption, ", (map {"$list[$_], "} (0 .. $size - 1)), "\n";
    }

    push @$output_data, format_list("avg", $size, @avg);
    push @$output_data, format_list("min", $size, @min);
    push @$output_data, format_list("min dev", $size, @min_dev);
    push @$output_data, format_list("max", $size, @max);
    push @$output_data, format_list("max dev", $size, @max_dev);


    # Make histogram data
    # I'm expecting 5 different bins:
    # mark4: 4 - 4.07
    # space4: 3.9 - 3.99
    # mark.5: .5 - .56
    # space1: .9 - 1
    # space2: 1.94 - 2

    for my $vector (@cooked_data)
    {
      for my $i (0 .. $size - 1)
      {
        my $value = $vector->[$i];
        my $binned = 0;
        for my $bin (keys %bins)
        {
          if (in_bin($bins{$bin}, $value))
          {
            $binned = 1;
            $bins{$bin}->{histogram_data}->{$value}++;
            push @{$bins{$bin}->{data}}, $value;
          }
        }

        if (!$binned)
        {
          push @$error, "error: value '$value' not binned\n";
        }
      }
    }
    push @$output_data, "\n";

    my @double_bits = ();
    for my $vector (@cooked_data)
    {
      # Now get strict with the data.  Everything has to bin appropriately.
      # Each vector should have $size elements.
      if (@$vector != $size)
      {
        push @$error, "vector has wrong size (expected $size, is ", 0 + @$vector, ")\n";
      }

      # 4ms mark.
      if (!in_bin($bins{mark_4ms}, $vector->[0]))
      {
        push @$error, "fail to bin to mark_4ms\n";
      }
      if (!in_bin($bins{space_4ms}, $vector->[1]))
      {
        push @$error, "fail to bin to space_4ms\n";
      }

      # Expect 24 2-tuple data bits, which can be either
      # "0": mark_500us, space_1ms
      # "1": mark_500us, space_2ms
      # ... and a final mark_500us.
      my @bitstring = ();
      my $i = 2;
      while ($i <= ($size - 2))
      {
        my $bit = undef;
        if (!in_bin($bins{mark_500us}, $vector->[$i]))
        {
          push @$error, "fail to bin to mark_500us\n";
        }
        $i++;

        if (in_bin($bins{space_1ms}, $vector->[$i]))
        {
          $bit = 0;
        }
        elsif (in_bin($bins{space_2ms}, $vector->[$i]))
        {
          $bit = 1;
        }
        else
        {
          push @$error, "data bit $i, $vector->[$i] didn't bin to space_1ms or space_2ms\n";
        }
        
        push @bitstring, $bit;
        $i++;
      }
      
      push @double_bits, [@bitstring];
    }
    push @$output_data, "\n";

    for my $bitlist (@double_bits)
    {
      if (@$bitlist != 24)
      {
        push @$error, "bitlist doesn't have 24 bits.\n";
      }

      for my $i (0 .. @$bitlist / 2 - 1)
      {
        push @$output_data, "$bitlist->[$i], ";
      }
      push @$output_data, "\n";
      for my $i (@$bitlist / 2 .. @$bitlist - 1)
      {
        my $complement = $bitlist->[$i - @$bitlist / 2] ? "0" : "1";
        if ($complement ne $bitlist->[$i])
        {
          push @$error, "complement mismatch\n";
        }
        push @$output_data, "$bitlist->[$i], ";
      }
      push @$output_data, "\n";
      push @$output_data, "\n";
    }

    # data bits are qualified as far as 12-bit data, 12-bit complemented data.
    # Print address, data representation in hex.
    # Data is MSB-first.
    # The first 4 bits are address, the remaining 8 bits are data.
    push @$output_data, "address, command\n";
    my %decoded_values = ();
    for my $bitlist (@double_bits)
    {
      my $address = 0;
      for my $i (0 .. 3)
      {
        $address *= 2;
        $address += $bitlist->[$i];
      }

      my $command = 0;
      for my $i (4 .. 11)
      {
        $command *= 2;
        $command += $bitlist->[$i];
      }
      push @$output_data, sprintf("0x%X, 0x%X\n", $address, $command);
      $decoded_values{sprintf("%01X%02X",$address, $command)} = 1;
    }
    push @$output_data, "\n";

    if ((keys %decoded_values) != 1)
    {
      push @$error, "decoded multiple values:\n";
      for my $key (keys %decoded_values)
      {
        push @$error, sprintf("0x%03X\n", $key);
      }
    }
    else
    {
      my ($decoded_value) = keys %decoded_values;
      $summary->{$button_name} = $decoded_value;
    }

    # Print binned data for histogram purposes.
    for my $bin (keys %bins)
    {
      push @$output_data, "$bin\n";
      my $data = $bins{$bin}->{histogram_data};
      for my $value (sort {$a <=> $b} keys %$data)
      {
        push @$output_data, "$value, $data->{$value}\n";
      }
      push @$output_data, "\n";
    }
    push @$output_data, "\n";
  }

  return ($summary, $output_data, $info, $error);
}

sub print_usage_and_exit
{
  print STDERR "Usage: $0 <input_dir> <output_dir>\n";
  exit 1;
}

if (@ARGV != 2)
{
  print_usage_and_exit();
}

my $input_dir = shift @ARGV;
my $output_dir = shift @ARGV;

if (!-d $input_dir)
{
  print STDERR "Can't find input dir '$input_dir'\n";
  exit 1;
}
if (!-d $output_dir)
{
  print STDERR "Can't find output dir '$output_dir'\n";
  exit 1;
}

opendir DIR, $input_dir || die "Can't open input dir '$input_dir'\n";
my @input_filenames = readdir DIR;
closedir DIR;

my %overall_summary = ();
for my $input_file_in_dir (@input_filenames)
{
  # Always, always must be skipping . and ..
  next if ($input_file_in_dir =~ /^\.{1,2}$/);

  my $input_filename = "$input_dir/$input_file_in_dir";
  if (!open INFILE, $input_filename)
  {
    print STDERR "Can't open file '$input_filename' for read\n";
    exit 1;
  }

  my @input = <INFILE>;
  close INFILE;

  my $output_filename = "$output_dir/$input_file_in_dir";
  if (!open OUTFILE, ">$output_filename")
  {
    print STDERR "Can't open file '$output_filename' for write\n";
    exit 1;
  }

  my ($summary, $output, $info, $error) = 
    transform_data(\@input);

  if (@$error)
  {
    print STDERR "$input_file_in_dir:\n", @$error;
    exit 1;
  }

  print STDERR "file: $input_filename\n";
  print STDERR @$info;
  print STDERR "\n";

  print OUTFILE @$output;
  close OUTFILE;
  for my $key (keys %$summary)
  {
    $overall_summary{$key} = $summary->{$key};
  }

}

my $summary_file = "$output_dir/summary.csv";
if (open FILE, ">$summary_file")
{
  print FILE "button, address, command\n";
  for my $button (sort {hex($overall_summary{$a}) <=> hex($overall_summary{$b})} keys %overall_summary)
  {
    my $value = hex($overall_summary{$button});
    my $address = ($value & 0xF00) >> 8;
    my $command = $value & 0xFF;
    printf FILE "$button, %01X, %02X\n", $address, $command;
  }

  close FILE;
}
else
{
  print STDERR "Failed to open summary file '$summary_file' for write\n";
}

my $html_summary = "$output_dir/table.html";
if (open FILE, ">$html_summary")
{
  # Header
  print FILE 
qq[<TABLE BORDER>
<TR ALIGN="RIGHT">
<TH>Button</TH>
<TH>Address (hex)</TH>
<TH>Command (hex)</TH>
</TR>
];

  # Per-row
  for my $button (sort {hex($overall_summary{$a}) <=> hex($overall_summary{$b})} keys %overall_summary)
  {
    my $value = hex($overall_summary{$button});
    my $address = sprintf("%01X", ($value & 0xF00) >> 8);
    my $command = sprintf("%02X", $value & 0xFF);

print FILE 
qq[<TR ALIGN="RIGHT">
<TD>$button</TD>
<TD>$address</TD>
<TD>$command</TD>
</TR>
];
  }

  print FILE "</TABLE>\n";
  close FILE;
}
else
{
  print STDERR "Failed to open html summary file '$html_summary' for write\n";
}

for my $bin (keys %bins)
{
  my $filename = "$output_dir/$bins{$bin}->{filename}";
  if (open FILE, ">$filename")
  {
    print FILE join("\n", @{$bins{$bin}->{data}});
    close FILE;
  }
  else 
  {
    print STDERR "Failed to open file '$filename' for write\n";
  }
}

exit 0;

__END__
:endofperl

